from os import makedirs, path
from typer import Argument, Exit
from rich.prompt import  Confirm

from nexy.cli.core.constants import CMD, Console
from nexy.cli.core.utils import generate_controller, generate_model, generate_service, print_banner  


RESOURCE_GENERATORS = {
    "controller": (generate_controller, "app/{name}", "controller.py"),
    "co": (generate_controller, "app/{name}", "controller.py"),
    "service": (generate_service, "app/{name}", "service.py"),
    "s": (generate_service, "app/{name}", "service.py"),
    "model": (generate_model, "app/{name}", "model.py"),
    "mo": (generate_model, "app/{name}", "model.py"),
}

# [Previous CLI commands remain the same until generate]

@CMD.command()
def generate(
    resource: str = Argument(..., help="Type de ressource à générer (controller/service/model)"),
    name: str = Argument(..., help="Nom de la ressource")
):
    """Génère un nouveau composant (controller, service, model)"""
    print_banner()
    
    if resource not in RESOURCE_GENERATORS:
        Console.print(f"[red]❌ Type de ressource invalide. Choix possibles: {', '.join(RESOURCE_GENERATORS.keys())}[/red]")
        raise Exit(1)
    
    generator_func, base_path, file_template = RESOURCE_GENERATORS[resource]
    
    # Vérifier si nous sommes dans un projet Nexy
    if not path.exists("app"):
        Console.print("[red]❌ Vous devez être dans un projet Nexy pour générer des composants.[/red]")
        raise Exit(1)
    
    # Créer le répertoire s'il n'existe pas
    makedirs(base_path, exist_ok=True)
    
    # Générer le fichier
    file_path = path.join(base_path, file_template.format(name=name.lower()))
    
    if path.exists(file_path):
        overwrite = Confirm.ask(
            f"[yellow]⚠️  {resource.capitalize()} {name} existe déjà. Voulez-vous l'écraser?[/yellow]"
        )
        if not overwrite:
            Console.print("[yellow]Génération annulée.[/yellow]")
            raise Exit(0)
    
    with open(file_path, "w") as f:
        f.write(generator_func(name))
    
    Console.print(f"[green]✨ {resource.capitalize()} {name} généré avec succès dans {file_path}[/green]")

@CMD.command()
def g(
    resource: str = Argument(..., help="Type de ressource à générer (raccourci pour generate)"),
    name: str = Argument(..., help="Nom de la ressource")
):
    """Alias raccourci pour generate"""
    generate(resource=resource, name=name)



from sys import platform
from typing import Optional
from typing_extensions import Annotated
from typer import Argument
from InquirerPy import inquirer

from nexy.cli.core.constants import Console, CMD
from nexy.cli.core.models import ORM, Database, ProjectType, TestFramework
from nexy.cli.core.project_builder import ProjectBuilder


def print_success_message(project_name: str, test_framework: TestFramework):
    """Affiche le message de succès après la création du projet"""
    success_message = f"""
[bold green]✨ Projet créé avec succès![/bold green]

Pour démarrer:
[yellow]cd {project_name}
{"./nexy_env/Scripts/activate" if platform == "win32" else "source nexy_env/bin/activate"}
nexy dev
[/yellow]
"""

    if test_framework != TestFramework.NONE:
        test_commands = {
            TestFramework.PYTEST: "pytest",
            TestFramework.UNITTEST: "python -m unittest discover tests",
            TestFramework.ROBOT: "robot tests/",
        }
        success_message += f"""
Pour lancer les tests:
[yellow]{test_commands[test_framework]}[/yellow]
"""

    Console.print(success_message)


def collect_project_options(builder: ProjectBuilder):
    """Collecte les options de configuration du projet via des prompts"""
    
    # Project Type
    project_type = ProjectType(inquirer.select(
        message="🤔 Started kit: ",
        choices=[t.value for t in ProjectType],
        default=ProjectType.API.value
    ).execute())
    builder.set_project_type(project_type)

    # Database
    database = Database(inquirer.select(
        message="Which database would you like to use: ",
        choices=[db.value for db in Database],
        default=Database.MYSQL.value
    ).execute())
    builder.set_database(database)

    # ORM
    if database != Database.NONE:
        orm = ORM(inquirer.select(
            message="Which ORM would you like to use: ",
            choices=[orm.value for orm in ORM],
            default=ORM.PRISMA.value
        ).execute())
        builder.set_orm(orm)

    # Test Framework
    test_framework = TestFramework(inquirer.select(
        message="Framework de test à utiliser:",
        choices=[tf.value for tf in TestFramework],
        height=20,
        default=TestFramework.PYTEST.value
    ).execute())
    builder.set_test_framework(test_framework)

    # Features
    if inquirer.confirm(message="Voulez-vous ajouter l'authentification?").execute():
        builder.add_feature("auth")
    if inquirer.confirm(message="Voulez-vous ajouter la validation des données?").execute():
        builder.add_feature("validation")
    if inquirer.confirm(message="Voulez-vous ajouter le support CORS?").execute():
        builder.add_feature("cors")
    if project_type == ProjectType.API and inquirer.confirm(
        message="Voulez-vous ajouter la documentation Swagger?"
    ).execute():
        builder.add_feature("swagger")

def create_project(project_name: Optional[str] = None):
    """Fonction commune pour créer un nouveau projet"""
    from nexy.cli.core.utils import print_banner
    
    print_banner()
    
    if not project_name:
        project_name = Console.input(f"[red]😡  your project missing name: [/red]")
    else:
        Console.print(f"✅  Project name:[green] {project_name}[/green]\n")

    # Créer et configurer le projet
    builder = ProjectBuilder(project_name)
    collect_project_options(builder)

    # Construire le projet
    Console.print("\n[bold green]Création du projet...[/bold green]")
    builder.build()

    # Afficher le message de succès
    print_success_message(project_name, builder.test_framework)




@CMD.command()
def new(project_name: Annotated[Optional[str], Argument(..., help="Nom du projet")] = None):
    """Crée un nouveau projet Nexy"""
    create_project(project_name)

@CMD.command()
def n(project_name: Annotated[Optional[str], Argument(..., help="Nom du projet")] = None):
    """Alias pour la commande new"""
    create_project(project_name)



from os import popen, path , getenv
from sys import version
from rich.table import Table

from nexy.cli.core.constants import Console, CMD
from nexy.cli.core.utils import print_banner


@CMD.command()
def info():
    """Affiche les informations sur le projet"""
    print_banner()
    
    table = Table(title="Information du Projet")
    table.add_column("Propriété", style="cyan")
    table.add_column("Valeur", style="magenta")
    
    try:
        with open("pyproject.toml", "r") as f:
            project_info = f.read()
    except FileNotFoundError:
        project_info = None
    
    table.add_row("Python Version", version.split()[0])
    table.add_row("Nexy Version", "1.0.0")
    table.add_row("Environnement", getenv("NEXY_ENV", "development"))
    
    if path.exists(".git"):
        git_branch = popen("git branch --show-current").read().strip()
        table.add_row("Git Branch", git_branch)
    
    if path.exists("requirements.txt"):
        with open("requirements.txt", "r") as f:
            deps = len(f.readlines())
            table.add_row("Dépendances", str(deps))
    
    Console.print(table)



from fastapi.responses import HTMLResponse
from jinja2 import Environment, FileSystemLoader, TemplateNotFound
import os

env = Environment(loader=FileSystemLoader("."), auto_reload=True)

def find_layouts(path):
    """Find all layout files in parent directories"""
    layouts = []
    current_path = path
    while current_path:
        layout_path = os.path.join(current_path, "layout.html")
        if os.path.exists(layout_path):
            layouts.insert(0, layout_path)
        current_path = os.path.dirname(current_path)
    return layouts

def useView(data, path):
    try:
        template_path = f"app{path}/view.html"
        
        # Find all layout templates in parent directories
        layouts = find_layouts(path)
        
        # If no layouts found, use default root layout
        if not layouts:
            root_layout = None
            try:
                root_layout = env.get_template("app/layout.html")
            except TemplateNotFound:
                # Create default root layout if not found
                layout_content = """
                <!DOCTYPE html>
                <html>
                <head>
                    <meta charset="utf-8">
                    <title>{% block title %}{% endblock %}</title>
                </head>
                <body>
                    {% block children %}{% endblock %}
                </body>
                </html>
                """
                root_layout = env.from_string(layout_content)
            layouts = [root_layout] if root_layout else []
        
        # Get the page template
        page = env.get_template(template_path)
        
        # Render page content within nested layouts
        content = page.render(data)
        for layout in layouts:
            layout_template = env.get_template(layout)
            content = layout_template.render(children=content, **data)
            
        return HTMLResponse(content=content)
    
    except TemplateNotFound as e:
        return HTMLResponse(content=f"Template not found: {str(e)}", status_code=404)
    except Exception as e:
        return HTMLResponse(content=f"Error rendering template: {str(e)}", status_code=500)



import os
import sys
import subprocess
from pathlib import Path

def install():
    """
    Installs Nexy dependencies using pip
    """
    try:
        # Check if requirements.txt exists
        if os.path.exists('requirements.txt'):
            subprocess.check_call([sys.executable, "-m", "pip", "install", "-r", "requirements.txt"])
            print("✨ Dependencies installed successfully!")
        else:
            print("❌ No requirements.txt file found")
            sys.exit(1)
    except subprocess.CalledProcessError as e:
        print("❌ Error installing dependencies:", e)
        sys.exit(1)


import os
import sys
import subprocess
from pathlib import Path

def upgrade():
    """
    Upgrades Nexy to the latest version using pip
    """
    try:
        subprocess.check_call([sys.executable, "-m", "pip", "install", "--upgrade", "nexy"])
        print("✨ Nexy has been upgraded successfully!")
    except subprocess.CalledProcessError as e:
        print("❌ Error upgrading Nexy:", e)
        sys.exit(1)



import importlib
from os import makedirs, name, path
import subprocess
from pathlib import Path
from socket import AF_INET, SOCK_STREAM, socket
import sys
from time import sleep
from typing import  List
from rich.columns import Columns
from rich.panel import Panel
from rich.progress import Progress, SpinnerColumn, TextColumn
from typer import Exit, echo

from nexy.cli.core.constants import Console
from nexy.cli.core.models import ORM, Database, ProjectType, TestFramework




def print_banner():
    Console.print(
"""
[green]
       _   __                
      / | / /__  _  ____  __
     /  |/ / _ \| |/_/ / / /
    / /|  /  __/>  </ /_/ / 
   /_/ |_/\___/_/|_|\__, /  
                   /____/   
[/green]
"""
)


# Fonctions utilitaires pour la gestion des ports
def is_port_in_use(port: int, host: str = 'http://localhost') -> bool:
    with socket(AF_INET, SOCK_STREAM) as s:
        try:
            s.bind((host, port))
            return False
        except OSError:
            return True

def find_available_port(start_port: int = 3000, host: str = 'http://localhost') -> list[int]:
    available_ports = []
    current_port = start_port
    while len(available_ports) < 5 and current_port < 65535:
        if not is_port_in_use(current_port, host):
            available_ports.append(current_port)
        current_port += 1
    return available_ports

def display_port_choices(available_ports: list[int], host: str) -> None:
    panels = []
    for i, port in enumerate(available_ports, 1):
        panel_content = f"[bold]Port {port}[/bold]\n{host}:{port}"
        color = ["green", "blue", "yellow", "cyan", "magenta"][i - 1]
        panels.append(Panel(
            panel_content,
            title=f"Choix {i}",
            border_style=color,
            padding=1
        ))
    Console.print(Columns(panels))




# Fonctions de génération de fichiers
def generate_requirements(project_type: ProjectType, database: Database, orm: ORM, 
                        test_framework: TestFramework, features: List[str]) -> str:
    requirements = [
        "nexy",
        "uvicorn",
        "python-dotenv",
        "inquirerpy==0.3.4"
    ]
    
    if database != Database.NONE:
        db_requirements = {
            Database.MYSQL: ["mysql-connector-python"],
            Database.POSTGRESQL: ["psycopg2-binary"],
            Database.MONGODB: ["motor"],
            Database.SQLITE: [],
        }
        requirements.extend(db_requirements.get(database, []))
    
    if orm != ORM.NONE:
        orm_requirements = {
            ORM.PRISMA: ["prisma"],
            ORM.SQLALCHEMY: ["sqlalchemy"],
        }
        requirements.extend(orm_requirements.get(orm, []))
    
    if test_framework != TestFramework.NONE:
        test_requirements = {
            TestFramework.PYTEST: ["pytest", "pytest-cov", "pytest-asyncio"],
            TestFramework.UNITTEST: ["unittest2"],
            TestFramework.ROBOT: ["robotframework"],
        }
        requirements.extend(test_requirements.get(test_framework, []))
    
    if project_type == ProjectType.WEBAPP:
        requirements.extend(["jinja2"])
    
    if "auth" in features:
        requirements.extend(["python-jose[cryptography]", "passlib[bcrypt]"])
    
    return "\n".join(requirements)

def generate_env_file(database: Database) -> str:
    env_vars = [
        "APP_ENV=development",
        "SECRET_KEY=your-secret-key-here",
    ]
    
    db_vars = {
        Database.MYSQL: ["DATABASE_URL=mysql://user:password@localhost:3306/dbname"],
        Database.POSTGRESQL: ["DATABASE_URL=postgresql://user:password@localhost:5432/dbname"],
        Database.MONGODB: ["DATABASE_URL=mongodb://localhost:27017/dbname"],
        Database.SQLITE: ["DATABASE_URL=sqlite:///./sql_app.db"],
    }
    
    if database != Database.NONE:
        env_vars.extend(db_vars.get(database, []))
    # 
    return "\n".join(env_vars)

def create_test_config(project_name: str, test_framework: TestFramework):
    if test_framework == TestFramework.PYTEST:
        pytest_ini = """[pytest]
testpaths = tests
python_files = test_*.py
python_classes = Test
python_functions = test_*
addopts = -v --cov=app
"""
        with open(path.join(project_name, "pytest.ini"), "w") as f:
            f.write(pytest_ini)
            
        test_example = """import pytest
from fastapi.testclient import TestClient
from main import app

client = TestClient(app)

def test_read_main():
    response = client.get("/")
    assert response.status_code == 200
"""
        makedirs(path.join(project_name, "tests"), exist_ok=True)
        with open(path.join(project_name, "tests", "test_main.py"), "w") as f:
            f.write(test_example)

    elif test_framework == TestFramework.ROBOT:
        robot_test = """*** Settings ***
Documentation     Example Test Suite
Library           RequestsLibrary

*** Test Cases ***
Test Main Page
    Create Session    app    http://localhost:3000
    ${response}=    GET On Session    app    /
    Status Should Be    200    ${response}
"""
        makedirs(path.join(project_name, "tests"), exist_ok=True)
        with open(path.join(project_name, "tests", "main.robot"), "w") as f:
            f.write(robot_test)

def setup_virtualenv(project_name: str, env_name: str, requirements_file: str = None):
    """Set up a virtual environment and install dependencies.
    
    Args:
        project_name: Name of the project directory
        env_name: Name of the virtual environment
        requirements_file: Optional path to requirements.txt file
    """
    try:
        venv_path = path.join(project_name, env_name)
        
        # Check if environment already exists
        if path.exists(venv_path):
            Console.print(f"[yellow]L'environnement {env_name} existe déjà.[/yellow]")
            return

        # Create virtual environment
        Console.print(f"[blue]Création de l'environnement virtuel {env_name}...[/blue]")
        result = subprocess.run(
            [sys.executable, '-m', 'venv', venv_path],
            capture_output=True,
            text=True,
            check=True
        )

        # Get pip path based on OS
        pip_executable = path.join(venv_path, 'Scripts', 'pip.exe') if sys.platform == "win32" else path.join(venv_path, 'bin', 'pip')
        python_executable = path.join(venv_path, 'Scripts', 'python.exe') if sys.platform == "win32" else path.join(venv_path, 'bin', 'python')
        
        # Upgrade pip using python -m pip to avoid permission issues
        Console.print("[bold yellow]Mise à jour de pip...[/bold yellow]")
        subprocess.run([python_executable, '-m', 'pip', 'install', '--upgrade', 'pip'], check=True)

        # Install dependencies if requirements file exists
        if requirements_file:
            Console.print(f"[bold yellow]Installation des dépendances depuis {requirements_file}...[/bold yellow]")
            subprocess.run([pip_executable, 'install', '-r', requirements_file], check=True)
            Console.print(f"[green]✓ Dépendances installées avec succès[/green]")

        Console.print(f"[green]✓ Environnement virtuel {env_name} créé avec succès[/green]")

    except subprocess.CalledProcessError as e:
        Console.print(f"[red]Erreur lors de l'exécution de la commande : {e.stderr}[/red]")
        raise Exit(1)
    except Exception as e:
        Console.print(f"[red]Une erreur est survenue : {str(e)}[/red]")
        raise Exit(1)



def create_project_structure(
    project_name: str,
    project_type: ProjectType,
    database: Database,
    orm: ORM,
    test_framework: TestFramework,
    features: List[str]
):
    base_dirs = [
        "",
        "app",
        "public",
        "tests",
        "config"
    ]
    
    if project_type == ProjectType.WEBAPP:
        base_dirs.extend(["app/templates", "app/static", "app/static/css", "app/static/js"])
    
    with Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        transient=True,
    ) as progress:
        progress.add_task("[green]Création de la structure du projet...[/green]", total=None)
        
        for dir_path in base_dirs:
            full_path = path.join(project_name, dir_path)
            makedirs(full_path, exist_ok=True)
            sleep(0.1)
        
        files_to_create = {
            "app/controller.py": """
async def GET():
    return {"name": "hello world"}
""",
            "requirements.txt": generate_requirements(project_type, database, orm, test_framework, features),
            ".env": generate_env_file(database),
            "README.md": generate_readme(project_name, project_type, database, orm, test_framework, features),
        }
        
        for file_name, content in files_to_create.items():
            with open(path.join(project_name, file_name), "w") as f:
                f.write(content)
            sleep(0.1)
        
        if test_framework != TestFramework.NONE:
            create_test_config(project_name, test_framework)

def generate_readme(project_name: str, project_type: ProjectType, database: Database, 
                   orm: ORM, test_framework: TestFramework, features: List[str]) -> str:
    testing_section = ""
    if test_framework != TestFramework.NONE:
        testing_commands = {
            TestFramework.PYTEST: "pytest",
            TestFramework.UNITTEST: "python -m unittest discover tests",
            TestFramework.ROBOT: "robot tests/",
        }
        test_command = testing_commands.get(test_framework, "")
        testing_section = f"""
## Tests
Pour exécuter les tests :
```bash
{test_command}
```
"""

    return f"""# {project_name}

## Description
Projet {project_type.value} généré avec Nexy CLI

## Configuration technique
- Type: {project_type.value}
- Base de données: {database.value}
- ORM: {orm.value}
- Framework de test: {test_framework.value}
- Fonctionnalités: {', '.join(features)}

## Installation

1. Cloner le projet
```bash
git clone <url-du-projet>
cd {project_name}
```

2. Installer les dépendances
```bash
pip install -r requirements.txt
```

3. Configurer les variables d'environnement
```bash
cp .env.example .env
# Modifier les variables dans .env
```

4. Lancer le serveur de développement
```bash
python main.py
```
{testing_section}
"""

# [Previous imports and code until the CLI commands remain the same...]

# Nouvelles fonctions pour la génération de composants
def generate_controller(name: str) -> str:
    return f"""from nexy.app import Controller

class {name.capitalize()}Controller(Controller):
    def __init__(self):
        super().__init__()
    
    async def index(self):
        return {{"message": "Welcome to {name} controller"}}
    
    async def get_by_id(self, id: int):
        return {{"id": id, "name": "{name}"}}
    
    async def create(self, data: dict):
        return {{"message": f"Created {name}", "data": data}}
    
    async def update(self, id: int, data: dict):
        return {{"message": f"Updated {name} {{id}}", "data": data}}
    
    async def delete(self, id: int):
        return {{"message": f"Deleted {name} {{id}}"}}
"""

def generate_service(name: str) -> str:
    return f"""class {name.capitalize()}Service:
    def __init__(self):
        # Initialize your service
        pass
    
    async def get_all(self):
        # Implement get all logic
        pass
    
    async def get_by_id(self, id: int):
        # Implement get by id logic
        pass
    
    async def create(self, data: dict):
        # Implement create logic
        pass
    
    async def update(self, id: int, data: dict):
        # Implement update logic
        pass
    
    async def delete(self, id: int):
        # Implement delete logic
        pass
"""

def generate_model(name: str) -> str:
    return f"""from sqlalchemy import Column, Integer, String, DateTime
from datetime import datetime
from nexy.app import Base

class {name.capitalize()}(Base):
    __tablename__ = "{name.lower()}s"

    id = Column(Integer, primary_key=True, index=True)
    name = Column(String, index=True)
    created_at = Column(DateTime, default=datetime.utcnow)
    updated_at = Column(DateTime, default=datetime.utcnow, onupdate=datetime.utcnow)
    
    def to_dict(self):
        return {{
            "id": self.id,
            "name": self.name,
            "created_at": self.created_at.isoformat(),
            "updated_at": self.updated_at.isoformat()
        }}
"""



def load_config():
    """
    Vérifie si config.py existe à la racine du projet et l'importe dynamiquement.
    """
    # Chemin du répertoire courant
    root_path = Path.cwd()
    
    # Chemin potentiel vers config.py
    config_file = root_path / "config.py"
    
    # Vérifier l'existence de config.py
    if not config_file.exists():
        echo("❌ Le fichier 'config.py' n'a pas été trouvé à la racine du projet.")
        raise Exit(code=1)
    
    # Importer dynamiquement config.py
    try:
        spec = importlib.util.spec_from_file_location("config", config_file)
        config = importlib.util.module_from_spec(spec)
        sys.modules["config"] = config
        spec.loader.exec_module(config)
        return config
    except Exception as e:
        echo(f"❌ Erreur lors du chargement de 'config.py' : {e}")
        raise Exit(code=1)

    




    


# Enums for different project options
from enum import Enum


class ProjectType(str, Enum):
    API = "API"
    WEBAPP = "Web Application"
    FULLSTACK = "Full Stack Application"
    MICROSERVICE = "Microservice"

class Database(str, Enum):
    MYSQL = "MySQL"
    POSTGRESQL = "PostgreSQL" 
    SQLITE = "SQLite"
    MONGODB = "MongoDB"
    REDIS = "Redis"
    NONE = "None"

class ORM(str, Enum):
    PRISMA = "Prisma"
    SQLALCHEMY = "SQLAlchemy"
    SQLMODEL = "SQLModel"
    NONE = "None"

class TestFramework(str, Enum):
    PYTEST = "Pytest"
    UNITTEST = "Unittest"
    ROBOT = "Robot Framework"
    NONE = "None"

class RouterType(str, Enum):
    APP_ROUTER = "App Router"
    MANUAL = "Manual Routing"

class AuthType(str, Enum):
    JWT = "JWT Authentication"
    SESSION = "Session Based"
    OAUTH = "OAuth 2.0"
    NONE = "None"

class CacheType(str, Enum):
    REDIS = "Redis"
    MEMCACHED = "Memcached"
    INMEMORY = "In-Memory"
    NONE = "None"

class Languages(str, Enum):
    ENGLISH = "English"
    FRENCH = "Français"
    SPANISH = "Español"
    ARABIC = "العربية"
    HINDI = "हिन्दी"
    CHINESE = "中文"
    PORTUGUESE = "Português"
    GERMAN = "Deutsch"



from os import system, path
from typer import Argument,Exit
from rich.progress import Progress, SpinnerColumn, TextColumn
import sys

from nexy.cli.core.constants import Console,CMD
from nexy.cli.core.utils import print_banner



@CMD.command()
def add(
    package: str = Argument(..., help="Package à ajouter au projet")
):
    """Ajoute une dépendance au projet"""
    print_banner()
    
    if not path.exists("requirements.txt"):
        Console.print("[yellow]⚠️  Fichier requirements.txt non trouvé. Création...[/yellow]\n")
    
    try:
        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
        ) as progress:
            task = progress.add_task(f"[green]Installation de {package}...", total=None)
            print("\n")
            system(f"pip install {package}")
            
            # Mettre à jour requirements.txt
            system(f"pip freeze > requirements.txt")
            
        Console.print(f"[green]✨ Package {package} installé et ajouté à requirements.txt[/green]\n")
    except Exception as e:
        Console.print(f"[red]❌ Erreur lors de l'installation: {str(e)}[/red]")
        raise Exit(1)



import os
from fastapi import APIRouter, Depends, FastAPI
from fastapi.responses import HTMLResponse
import sys

from nexy.hooks import useView
from .utils import deleteFistDotte, dynamicRoute, importModule, convertPathToModulePath

# Analyze the file structure and extract route information
def FIND_ROUTES(base_path):
    routes: list = []
    
    # Verify if the 'app' folder exists
    if os.path.exists(base_path) and os.path.isdir(base_path):
        # Add app directory to Python path
        app_dir = os.path.abspath(base_path)
        if app_dir not in sys.path:
            sys.path.append(app_dir)
            
        # Explore the 'app' folder and its subfolders
        for root, dirs, files in os.walk(base_path):
            # Remove _folders
            dirs[:] = [d for d in dirs if not d.startswith("_")]

            route = {
                "pathname": f"{'/' if os.path.basename(root) == base_path else '/' + deleteFistDotte(os.path.relpath(root, base_path).replace('\\','/'))}",
                "dirName": root
            }
            controller = os.path.join(root, 'controller.py')
            middleware = os.path.join(root, 'middleware.py')
            service = os.path.join(root, 'service.py')

            # Check for files and add to dictionary
            if os.path.exists(controller):
                route["controller"] = convertPathToModulePath(f"{root}/controller")    
            if os.path.exists(middleware):
                route["middleware"] = convertPathToModulePath(f"{root}/middleware") 
            if os.path.exists(service):
                route["service"] = convertPathToModulePath(f"{root}/service") 
            routes.append(route)

    return routes


def Router(app: FastAPI):
    """
    Dynamically loads routes from the 'app' directory.
    """
    
    routes = FIND_ROUTES(base_path="app")
    HTTP_METHODES: tuple = ["GET", "POST", "PUT", "DELETE", "OPTIONS", "HEAD", "PATCH", "TRACE"]
    
    for route in routes:

        pathname = dynamicRoute(route_in=route["pathname"])
        
        if "controller" in route:
            try:
                # Import the controller module
                try:
                    controller = importModule(path=route["controller"])
                except ModuleNotFoundError as e:
                    print(f"Error loading No Found {route['controller']}: {e}")
                    continue
                except Exception as e:
                    print(f"Error loading module {route['controller']}: {str(e)}")
                    continue
                
                for function_name in dir(controller):
                    function = getattr(controller, function_name)
                    
                    # Check if function is callable and has annotations
                    if callable(function) and hasattr(function, "__annotations__"):
                        params = getattr(function, "params", {})

                        # Add route for each HTTP method
                        if function_name in HTTP_METHODES:
                            try:

                                if params.get("response_class") == HTMLResponse:
                                    def view( data =  Depends(function)):
                                        return useView( data=data,path=f"{pathname}")
                                    app.add_api_route(
                                        path=pathname,
                                        endpoint= view,
                                        methods=[function_name], 
                                        **{key: value for key, value in params.items() if key != "tags"},
                                        tags=[pathname]
                                    )
                                else:
                                    app.add_api_route(
                                        path=pathname,
                                        endpoint=function,
                                        methods=[function_name],
                                        **{key: value for key, value in params.items() if key != "tags"},
                                        tags=[pathname]
                                    )
                                
                            except Exception as e:
                                # Handle route addition errors
                                print(f"Error adding route {pathname} for method {function_name}: {e}")
                                app.add_api_route(
                                    path=pathname,
                                    endpoint=lambda: {"error": f"Error in method {function_name} for route {pathname}. {str(e)}"},
                                    methods=[function_name],
                                    status_code=500
                                )

                        # Add WebSocket route if Socket method exists
                        elif function_name == "SOCKET":
                            try:
                                app.add_api_websocket_route(f"{pathname}/ws", function)
                            except Exception as e:
                                print(f"Error adding WebSocket for route {pathname}: {e}")
                                app.add_api_websocket_route(
                                    f"{pathname}/ws",
                                    lambda: {"error": f"Error in WebSocket connection for route {pathname}. {str(e)}"}
                                )

            except Exception as e:
                # Handle module import errors
                print(f"Error loading module {route['controller']}: {e}")
                app.add_api_route(
                    path=pathname,
                    endpoint=lambda: {"error": f"Error loading module {route['controller']}. {str(e)}"},
                    methods=["GET"],
                    status_code=500
                )
    return app



from  os import system, path
from sys import platform
import typer
from rich.prompt import  IntPrompt


from nexy.cli.core.constants import Console,CMD
# from nexy.cli.core.utils import display_port_choices, find_available_port, load_config, is_port_in_use, print_banner


try:
    from nexy import Nexy
    app = Nexy()
        
except ImportError:  # pragma: no cover
    pass


# module = load_config()

# NEXY_CONFIGS = module.NEXY_CONFIGS or None
  

@CMD.command()
def dev(
    port: int = typer.Option(3000, "--port", "-p", help="Port du serveur"),
    host: str = typer.Option("localhost", "--host", help="Host du serveur")
)-> None:
    """Lance le serveur de développement"""
    # print(NEXY_CONFIGS.PORT)
        
    Console.print(Console.print(f"[green]🚀 Démarrage du serveur sur http://{host}:{port}[/green]"))
    
    system(f"uvicorn nexy.cli.commands.dev:app --host {host} --port {port} --reload --log-level debug  --use-colors ")
    

def add(package: str):
    # Devrait utiliser le pip de l'environnement virtuel nexy_env
    if path.exists("nexy_env"):
        pip_path = "nexy_env/Scripts/pip" if platform == "win32" else "nexy_env/bin/pip"
        system(f"{pip_path} install {package}")
    else:
        system(f"pip install {package}")




"""Nexy a Python framework designed to combine simplicity, performance, and the joy of development."""

__version__ = "0.0.28.3"

from nexy.decorators import Injectable as Injectable
from nexy.decorators import Config as Config
from nexy.decorators import Inject as Inject
from nexy.decorators import CustomResponse as CustomResponse
from nexy.decorators import Describe as Describe
from nexy.app import Nexy as Nexy
from nexy.hooks import *

from fastapi import BackgroundTasks as BackgroundTasks
from fastapi import Depends as Depends
from fastapi import Body as Body
from fastapi import Cookie as Cookie
from fastapi import File as File
from fastapi import Form as Form
from fastapi import Header as Header
from fastapi import Query as Query
from fastapi import Security as Security
from fastapi import HTTPException as HTTPException
from fastapi import Path as Path
from fastapi import Request as Request
from fastapi import WebSocket as WebSocket
from fastapi import WebSocketException as WebSocketException
from fastapi import WebSocketDisconnect as WebSocketDisconnect
from fastapi import UploadFile as UploadFile

from fastapi.responses import FileResponse as FileResponse
from fastapi.responses import HTMLResponse as HTMLResponse
from fastapi.responses import JSONResponse as JSONResponse
from fastapi.responses import ORJSONResponse as ORJSONResponse
from fastapi.responses import PlainTextResponse as PlainTextResponse
from fastapi.responses import RedirectResponse as RedirectResponse
from fastapi.responses import Response as Response


__all__ = [
    # Nexy-related exports
    "Nexy",
    "Injectable",
    "Config", 
    "Inject",
    "CustomResponse",
    "Describe",
    
    # FastAPI responses
    "Response",
    "FileResponse",
    "HTMLResponse", 
    "JSONResponse",
    "ORJSONResponse",
    "PlainTextResponse",
    "RedirectResponse",
    
    # FastAPI utilities
    "BackgroundTasks",
    "Depends",
    "Body",
    "Cookie", 
    "File",
    "Form",
    "Header",
    "Query",
    "Security",
    "HTTPException",
    "Path",
    "Request",
    "WebSocket",
    "WebSocketException", 
    "WebSocketDisconnect",
    "UploadFile",
]
